﻿Public Class Graphs

  Dim MousePress As Boolean = False
  Dim recalculateflag As Boolean = False

  Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
    CoreMass = NumericUpDown1.Value * SolarMass * 1000000000
    SetGalacticRotationParas() : SetDiscDensity() : SetHaloDensity()
    recalculateflag = True
    Recalculate() : UpdateGraphs()
  End Sub

  Private Sub NumericUpDown1_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NumericUpDown1.ValueChanged
    CoreMass = NumericUpDown1.Value * SolarMass * 1000000000
    Recalculate() : UpdateGraphs()
  End Sub

  Private Sub SetDiscDensity()
    Dim Scale As Double = 1.0E-21
    For i As Integer = 0 To 40
      Select Case DiscModel
        Case "Flat"
          If i > NumericUpDown10.Value Then
            DiscDensity(i) = 0
          Else
            DiscDensity(i) = NumericUpDown2.Value * Scale
          End If
        Case "Linear"
          DiscDensity(i) = (NumericUpDown11.Value - i) * NumericUpDown3.Value * Scale / NumericUpDown11.Value
          If DiscDensity(i) < 0 Then DiscDensity(i) = 0
        Case "Reciprocal"
          If i = 0 Then DiscDensity(i) = NumericUpDown4.Value * Scale Else DiscDensity(i) = NumericUpDown4.Value * Scale / i
        Case "Exponential"
          DiscDensity(i) = NumericUpDown14.Value * NumericUpDown5.Value ^ (-i) * Scale
      End Select
    Next
  End Sub

  Private Sub SetHaloDensity()
    Dim Scale As Double = 1.0E-21
    For i As Integer = 0 To 40
      Select Case HaloModel
        Case "Flat"
          If i > NumericUpDown13.Value Then
            HaloDensity(i) = 0
          Else
            HaloDensity(i) = NumericUpDown9.Value * Scale
          End If
        Case "Reciprocal"
          If i = 0 Then
            HaloDensity(i) = NumericUpDown8.Value * Scale
          Else
            HaloDensity(i) = NumericUpDown8.Value * Scale / i
          End If
        Case "Inverse square"
          If i = 0 Then
            HaloDensity(i) = NumericUpDown7.Value * Scale
          Else
            HaloDensity(i) = NumericUpDown7.Value * Scale / i / i
          End If
        Case "Exponential"
          HaloDensity(i) = NumericUpDown15.Value * NumericUpDown6.Value ^ (-i) * Scale
      End Select
    Next
  End Sub

  Private Sub Recalculate()
    If recalculateflag = False Then Exit Sub
    For i As Integer = 0 To 40
      GFS(0, i) = CalculateCoreGFS(i * kPc)
      GFS(1, i) = CalculateDiscGFS(i * kPc) ' GFS in m/s2
      GFS(2, i) = CalculateHaloGFS(i * kPc)
      GFS(3, i) = GFS(0, i) + GFS(1, i) + GFS(2, i)
      If GFS(3, i) > 0 Then
        RotSpeed(i) = Math.Sqrt(i * kPc * GFS(3, i))
      Else
        RotSpeed(i) = 0
      End If
    Next
    Label7.Text = NumericUpDown1.Value
    Dim mc As Double = NumericUpDown1.Value
    Dim md As Double = 0
    Dim mh As Double = 0
    For i As Integer = 1 To 40
      md += DiscDensity(i) * 2 * Math.PI * (i + 0.5)
      mh += HaloDensity(i) * 4 * Math.PI * i * i
    Next
    md = md * kPc * kPc * kPc / SolarMass / 1000000000
    mh = mh * kPc * kPc * kPc / SolarMass / 1000000000
    Label8.Text = md.ToString("F0")
    Label11.Text = mh.ToString("F0")
    Dim mt As Double = mc + md + mh
    Label18.Text = mt.ToString("F0")
    Dim ratio As Integer
    If mt = 0 Then ratio = 0 Else ratio = CInt(100 * mh / mt)
    Label14.Text = ratio.ToString("F0")
  End Sub

  Private Function CalculateCoreGFS(ByVal r As Double) As Double
    If r = 0 Then Return 0
    Return Gconst * CoreMass / (r * r)
  End Function

  Private Function CalculateHaloGFS(ByVal r As Double) As Double
    If r = 0 Then Return 0
    Dim g As Double = 0
    Dim n As Integer = r / kPc + 0.1
    For i As Integer = 1 To n
      g += i * i * HaloDensity(i)
    Next
    g = 4 * Math.PI * Gconst * kPc * kPc * kPc * g / (r * r)
    Return g
  End Function

  Private Function CalculateDiscGFS(ByVal r As Double) As Double
    Dim a, g As Double
    If r = 0 Then Return 0
    g = 0
    For i As Integer = 0 To 40
      a = (i + 0.5) * kPc ' a in metres
      g += CalculateRingGravity(a, r, DiscDensity(i))
    Next
    Return g ' GFS in m/s2
  End Function

  Private Function CalculateRingGravity(ByVal a As Double, ByVal r As Double, ByVal rho As Double) As Double
    Dim theta, COSalpha, d, dg, g, m As Double
    ' each ring is assumed to be 1 square kPc in cross section
    m = 2 * Math.PI * a * kPc * kPc * rho / Nseg ' mass of segment
    g = 0
    For i As Integer = 0 To Nseg - 1
      theta = (i + 0.5) * Math.PI / 10 ' offset by a small amount so that we avoid infinities
      d = Math.Sqrt(r * r + a * a - 2 * r * a * Math.Cos(theta))
      COSalpha = (r * r + d * d - a * a) / (2 * r * d)
      dg = Gconst * m * COSalpha / (d * d)
      g += dg
    Next
    Return g ' GFS in m/s2
  End Function

  Private Sub PictureBox1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles PictureBox1.Paint
    Dim g As Graphics = e.Graphics
    g.TranslateTransform(26, 180)
    g.DrawString("0", Font, Brushes.Black, -12, -5)
    g.DrawString("5", Font, Brushes.Black, -12, -85)
    g.DrawString("10", Font, Brushes.Black, -18, -165)
    Xaxisnumbers(g)
    g.ScaleTransform(1, -1)
    g.DrawLine(Pens.Black, 0, 0, 560, 0)
    g.DrawLine(Pens.Black, 0, 0, 0, 160)
    Xaxis(g, 40)
    Yaxis(g, 10)
    g.SetClip(New Rectangle(0, 0, 560, 160))
    Dim p As New Pen(Color.DarkGreen, 2)
    For i As Integer = 1 To 40
      g.DrawLine(p, 14 * (i - 1), CInt(16 * DiscDensity(i - 1) * 1.0E+21), 14 * i, CInt(16 * DiscDensity(i) * 1.0E+21))
    Next
    p = New Pen(Color.DarkBlue, 2)
    For i As Integer = 1 To 40
      g.DrawLine(p, 14 * (i - 1), CInt(16 * HaloDensity(i - 1) * 1.0E+21), 14 * i, CInt(16 * HaloDensity(i) * 1.0E+21))
    Next
    DensityDataPoint(g, LocalDensity)
  End Sub

  Private Sub DensityDataPoint(ByVal g As Graphics, ByVal d As DataPoint)
    Dim x, y1, y2 As Integer
    x = 14 * d.x : y1 = 16 * (d.y + d.e) : y2 = 16 * (d.y - d.e)
    g.DrawLine(Pens.Green, x, y1, x, y2)
    g.DrawLine(Pens.Green, x - 4, y1, x + 4, y1)
    g.DrawLine(Pens.Green, x - 4, y2, x + 4, y2)
  End Sub

  Private Sub PictureBox2_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles PictureBox2.Paint
    Dim g As Graphics = e.Graphics
    g.TranslateTransform(26, 180)
    g.DrawString("0", Font, Brushes.Black, -12, -5)
    g.DrawString("20", Font, Brushes.Black, -18, -85)
    g.DrawString("40", Font, Brushes.Black, -18, -165)
    Xaxisnumbers(g)
    g.ScaleTransform(1, -1)
    g.DrawLine(Pens.Black, 0, 0, 560, 0)
    g.DrawLine(Pens.Black, 0, 0, 0, 160)
    Xaxis(g, 40)
    Yaxis(g, 10)
    g.SetClip(New Rectangle(0, 0, 560, 160))
    DrawGFSline(g, 0, Color.Black)
    DrawGFSline(g, 1, Color.DarkGreen)
    DrawGFSline(g, 2, Color.DarkBlue)
    DrawGFSline(g, 3, Color.Red)
  End Sub

  Private Sub DrawGFSline(ByVal g As Graphics, ByVal n As Integer, ByVal colour As Color)
    Dim p As New Pen(colour, 2)
    For i As Integer = 2 To 40
      g.DrawLine(p, 14 * (i - 1), CInt(4 * GFS(n, i - 1) * 100000000000.0), 14 * i, CInt(4 * GFS(n, i) * 100000000000.0))
    Next
  End Sub

  Private Sub PictureBox3_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles PictureBox3.Paint
    Dim g As Graphics = e.Graphics
    g.TranslateTransform(26, 180)
    g.DrawString("0", Font, Brushes.Black, -12, -5)
    g.DrawString("200", Font, Brushes.Black, -24, -85)
    g.DrawString("400", Font, Brushes.Black, -24, -165)
    Xaxisnumbers(g)
    g.ScaleTransform(1, -1)
    g.DrawLine(Pens.Black, 0, 0, 560, 0)
    g.DrawLine(Pens.Black, 0, 0, 0, 160)
    Xaxis(g, 40)
    Yaxis(g, 10)
    g.SetClip(New Rectangle(0, 0, 560, 160))
    Dim p As New Pen(Color.DarkRed, 2)
    For i As Integer = 2 To 40
      g.DrawLine(p, 14 * (i - 1), CInt(40 * RotSpeed(i - 1) / 100000), 14 * i, CInt(40 * RotSpeed(i) / 100000))
    Next
    Select Case True
      Case RadioButton6.Checked
        For i As Integer = 0 To mwgn
          DataPoint(g, MWG(i))
        Next
      Case RadioButton7.Checked
        For i As Integer = 0 To M31N
          DataPoint(g, M31(i))
        Next
      Case RadioButton13.Checked
        For i As Integer = 0 To NGC3198N
          DataPoint(g, NGC3198(i))
        Next
    End Select
  End Sub

  Private Sub DataPoint(ByVal g As Graphics, ByVal d As DataPoint)
    Dim x, y1, y2 As Integer
    x = 14 * d.x : y1 = 40 * (d.y + d.e) / 100 : y2 = 40 * (d.y - d.e) / 100
    g.DrawLine(Pens.Red, x, y1, x, y2)
    g.DrawLine(Pens.Red, x - 4, y1, x + 4, y1)
    g.DrawLine(Pens.Red, x - 4, y2, x + 4, y2)
  End Sub

  Private Sub Xaxisnumbers(ByVal g)
    For i As Integer = 0 To 40 Step 10
      g.DrawString(Str(i), Font, Brushes.Black, 14 * i - 8, 0)
    Next
  End Sub

  Private Sub Xaxis(ByVal g, ByVal n)
    Dim d As Integer = 560 / n
    For i As Integer = 1 To n
      g.DrawLine(Pens.LightGray, d * i, 0, d * i, 160)
    Next
    Dim p As New Pen(Color.LightGray, 2)
    For i As Integer = 10 To n Step 10
      g.DrawLine(p, d * i, 0, d * i, 160)
    Next
    p = New Pen(Color.Black, 2)
    g.DrawLine(p, 0, 0, 0, 160)
  End Sub

  Private Sub Yaxis(ByVal g, ByVal n)
    Dim d As Integer = 160 / n
    For i As Integer = 1 To n
      g.DrawLine(Pens.LightGray, 0, d * i, 560, d * i)
    Next
    Dim p As New Pen(Color.Black, 2)
    g.DrawLine(p, 0, 0, 560, 0)
  End Sub

  Private Sub UpdateGraphs()
    PictureBox1.Invalidate()
    PictureBox2.Invalidate()
    PictureBox3.Invalidate()
  End Sub

  Private Sub RadioButton1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles RadioButton1.CheckedChanged, RadioButton2.CheckedChanged, RadioButton3.CheckedChanged, _
  RadioButton4.CheckedChanged, RadioButton5.CheckedChanged
    If sender.ToString.EndsWith("False") Then Exit Sub
    Select Case True
      Case RadioButton1.Checked : DiscModel = "Flat"
      Case RadioButton2.Checked : DiscModel = "Linear"
      Case RadioButton3.Checked : DiscModel = "Reciprocal"
      Case RadioButton4.Checked : DiscModel = "Exponential"
      Case RadioButton5.Checked : DiscModel = "Custom"
    End Select
    SetDiscDensity() : Recalculate() : UpdateGraphs()
  End Sub

  Private Sub NumericUpDown2_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles NumericUpDown2.ValueChanged, NumericUpDown3.ValueChanged, NumericUpDown4.ValueChanged, _
  NumericUpDown5.ValueChanged, NumericUpDown10.ValueChanged, NumericUpDown11.ValueChanged, _
   NumericUpDown14.ValueChanged
    SetDiscDensity() : Recalculate() : UpdateGraphs()
  End Sub

  Private Sub NumericUpDown6_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles NumericUpDown6.ValueChanged, NumericUpDown7.ValueChanged, NumericUpDown8.ValueChanged, _
  NumericUpDown9.ValueChanged, NumericUpDown13.ValueChanged, _
  NumericUpDown15.ValueChanged
    SetHaloDensity() : Recalculate() : UpdateGraphs()
  End Sub

  Private Sub RadioButton8_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles RadioButton8.CheckedChanged, RadioButton9.CheckedChanged, RadioButton10.CheckedChanged, _
  RadioButton11.CheckedChanged, RadioButton12.CheckedChanged
    If sender.ToString.EndsWith("False") Then Exit Sub
    Select Case True
      Case RadioButton12.Checked : HaloModel = "Flat"
      Case RadioButton11.Checked : HaloModel = "Reciprocal"
      Case RadioButton10.Checked : HaloModel = "Inverse square"
      Case RadioButton9.Checked : HaloModel = "Exponential"
      Case RadioButton8.Checked : HaloModel = "Custom"
    End Select
    SetHaloDensity() : Recalculate() : UpdateGraphs()
  End Sub

  Private Sub PictureBox1_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles PictureBox1.MouseMove
    If MousePress Then AddPoint(e.X, e.Y, e.Button)
  End Sub

  Private Sub PictureBox1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) _
  Handles PictureBox1.MouseDown
    MousePress = True : AddPoint(e.X, e.Y, e.Button)
  End Sub

  Private Sub PictureBox1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) _
  Handles PictureBox1.MouseUp
    MousePress = False
  End Sub

  Private Sub AddPoint(ByVal x, ByVal y, ByVal b)
    Dim i As Integer = Limit((x - 24) / 14, 0, 40)
    Dim d As Integer = Limit((180 - y) / 16, 0, 10)
    Dim Scale As Double = 1.0E-21
    If DiscModel = "Custom" And b = &H100000 Then DiscDensity(i) = d * Scale : Recalculate() : UpdateGraphs()
    If HaloModel = "Custom" And b = &H200000 Then HaloDensity(i) = d * Scale : Recalculate() : UpdateGraphs()
  End Sub

  Private Function Limit(ByVal a, ByVal b, ByVal c)
    If a < b Then Return b
    If a > c Then Return c
    Return a
  End Function

  Private Sub RadioButton6_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) _
  Handles RadioButton6.CheckedChanged, RadioButton7.CheckedChanged, RadioButton13.CheckedChanged
    UpdateGraphs()
  End Sub

  Private Sub Button1_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Button1.Click
    Help.ShowHelp(Me, Application.StartupPath & "\Galactic Rotation Curves.chm", HelpNavigator.Topic, "Introduction.html")
  End Sub
End Class